@@ -27,15 +27,22 @@ def is_databricks_uri(uri):
     return scheme == 'databricks' or uri == 'databricks'
 
 
-def get_db_profile_from_uri(uri):
+def construct_db_uri_from_profile(profile):
+    if profile:
+        return 'databricks://' + profile
+
+
+def get_db_info_from_uri(uri):
     """
     Get the Databricks profile specified by the tracking URI (if any), otherwise
     returns None.
     """
     parsed_uri = urllib.parse.urlparse(uri)
     if parsed_uri.scheme == "databricks":
-        return parsed_uri.netloc
-    return None
+        parsed_path = parsed_uri.path.lstrip('/') or None
+        parsed_profile = parsed_uri.netloc
+        return parsed_profile, parsed_path
+    return None, None
 
 
 def extract_db_type_from_uri(db_uri):
@@ -67,6 +74,12 @@ def get_uri_scheme(uri_or_path):
         return scheme
 
 
+def extract_and_normalize_path(uri):
+    parsed_uri_path = urllib.parse.urlparse(uri).path
+    normalized_path = posixpath.normpath(parsed_uri_path)
+    return normalized_path.lstrip("/")
+
+
 def append_to_uri_path(uri, *paths):
     """
     Appends the specified POSIX `paths` to the path component of the specified `uri`.
@@ -129,3 +142,9 @@ def _join_posixpaths_and_append_absolute_suffixes(prefix_path, suffix_path):
     # joined path
     suffix_path = suffix_path.lstrip(posixpath.sep)
     return posixpath.join(prefix_path, suffix_path)
+
+
+def is_databricks_acled_artifacts_uri(artifact_uri):
+    _ACLED_ARTIFACT_URI = "databricks/mlflow-tracking/"
+    artifact_uri_path = extract_and_normalize_path(artifact_uri)
+    return artifact_uri_path.startswith(_ACLED_ARTIFACT_URI)
